@@ -7,9 +7,8 @@ from todoist.managers.biz_invitations import BizInvitationsManager
 from todoist.managers.filters import FiltersManager
 from todoist.managers.invitations import InvitationsManager
 from todoist.managers.live_notifications import LiveNotificationsManager
-from todoist.managers.notes import NotesManager
+from todoist.managers.notes import NotesManager, ProjectNotesManager
 from todoist.managers.projects import ProjectsManager
-from todoist.managers.project_notes import ProjectNotesManager
 from todoist.managers.items import ItemsManager
 from todoist.managers.labels import LabelsManager
 from todoist.managers.reminders import RemindersManager
@@ -36,7 +35,7 @@ class TodoistAPI(object):
                 setattr(obj, key, data[key])
         return obj
 
-    def __init__(self, token='', api_endpoint='https://api.todoist.com'):
+    def __init__(self, token='', api_endpoint='https://api.todoist.com', session=None):
         self.api_endpoint = api_endpoint
         self.seq_no = 0  # Sequence number since last update
         self.seq_no_partial = {}  # Sequence number of partial syncs
@@ -66,6 +65,7 @@ class TodoistAPI(object):
         self.token = token  # User's API token
         self.temp_ids = {}  # Mapping of temporary ids to real ids
         self.queue = []  # Requests to be sent are appended here
+        self.session = session or requests.Session()  # Session instance for requests
 
         # managers
         self.projects = ProjectsManager(self)
@@ -221,7 +221,6 @@ class TodoistAPI(object):
         Updates the seq_no and the seq_no_partial, based on the seq_no in
         the response and the resource_types that were requested.
         """
-        print(seq_no, seq_no_global, resource_types)
         if not seq_no and not seq_no_global or not resource_types:
             return
         if 'all' in resource_types:
@@ -247,8 +246,8 @@ class TodoistAPI(object):
         """
         # Go through all the objects for which we expect the temporary id to be
         # replaced by a real one.
-        for datatype in ['Filters', 'Items', 'Labels', 'Notes', 'Projects',
-                         'Reminders']:
+        for datatype in ['Filters', 'Items', 'Labels', 'Notes', 'ProjectNotes',
+                         'Projects', 'Reminders']:
             for obj in self.state[datatype]:
                 if obj.temp_id == temp_id:
                     obj['id'] = new_id
@@ -262,7 +261,9 @@ class TodoistAPI(object):
         """
         if not url:
             url = self.get_api_url()
-        response = requests.get(url + call, **kwargs)
+
+        response = self.session.get(url + call, **kwargs)
+
         try:
             return response.json()
         except ValueError:
@@ -275,12 +276,23 @@ class TodoistAPI(object):
         """
         if not url:
             url = self.get_api_url()
-        response = requests.post(url + call, **kwargs)
+
+        response = self.session.post(url + call, **kwargs)
+
         try:
             return response.json()
         except ValueError:
             return response.text
 
+    def _json_serializer(self, obj):
+        import datetime
+        if isinstance(obj, datetime.datetime):
+            return obj.strftime('%Y-%m-%dT%H:%M:%S')
+        elif isinstance(obj, datetime.date):
+            return obj.strftime('%Y-%m-%d')
+        elif isinstance(obj, datetime.time):
+            return obj.strftime('%H:%M:%S')
+
     # Sync
     def generate_uuid(self):
         """
@@ -293,20 +305,22 @@ class TodoistAPI(object):
         Sends to the server the changes that were made locally, and also
         fetches the latest updated data from the server.
         """
-        params = {
+        data = {
             'token': self.token,
-            'commands': json.dumps(commands or []),
+            'commands': json.dumps(commands or [], separators=',:',
+                                   default=self._json_serializer),
             'day_orders_timestamp': self.state['DayOrdersTimestamp'],
         }
         if not commands:
-            params['seq_no'], params['seq_no_global'] = \
+            data['seq_no'], data['seq_no_global'] = \
                 self._get_seq_no(kwargs.get('resource_types', None))
 
         if 'include_notification_settings' in kwargs:
-            params['include_notification_settings'] = 1
+            data['include_notification_settings'] = 1
         if 'resource_types' in kwargs:
-            params['resource_types'] = json.dumps(kwargs['resource_types'])
-        data = self._post('sync', params=params)
+            data['resource_types'] = json.dumps(kwargs['resource_types'],
+                                                separators=',:')
+        data = self._post('sync', data=data)
         self._update_state(data)
         if not commands:
             self._update_seq_no(data.get('seq_no', None),
@@ -339,8 +353,8 @@ class TodoistAPI(object):
         """
         Logins user, and returns the response received by the server.
         """
-        data = self._get('login', params={'email': email,
-                                          'password': password})
+        data = self._post('login', data={'email': email,
+                                         'password': password})
         if 'token' in data:
             self.token = data['token']
         return data
@@ -351,9 +365,9 @@ class TodoistAPI(object):
         the server.
 
         """
-        params = {'email': email, 'oauth2_token': oauth2_token}
-        params.update(kwargs)
-        data = self._get('login_with_google', params=params)
+        data = {'email': email, 'oauth2_token': oauth2_token}
+        data.update(kwargs)
+        data = self._post('login_with_google', data=data)
         if 'token' in data:
             self.token = data['token']
         return data
@@ -363,29 +377,40 @@ class TodoistAPI(object):
         """
         Registers a new user.
         """
-        params = {'email': email, 'full_name': full_name, 'password': password}
-        params.update(kwargs)
-        data = self._get('register', params=params)
+        data = {'email': email, 'full_name': full_name, 'password': password}
+        data.update(kwargs)
+        data = self._post('register', data=data)
         if 'token' in data:
             self.token = data['token']
         return data
 
+    def delete_user(self, current_password, **kwargs):
+        """
+        Deletes an existing user.
+        """
+        params = {'token': self.token,
+                  'current_password': current_password}
+        params.update(kwargs)
+        return self._get('delete_user', params=params)
+
     # Miscellaneous
     def upload_file(self, filename, **kwargs):
         """
         Uploads a file.
         """
-        params = {'token': self.token}
-        params.update(kwargs)
+        data = {'token': self.token}
+        data.update(kwargs)
         files = {'file': open(filename, 'rb')}
-        return self._post('upload_file', self.get_api_url(), params=params,
+        return self._post('upload_file', self.get_api_url(), data=data,
                           files=files)
 
     def query(self, queries, **kwargs):
         """
         Performs date queries and other searches, and returns the results.
         """
-        params = {'queries': json.dumps(queries), 'token': self.token}
+        params = {'queries': json.dumps(queries, separators=',:',
+                                        default=self._json_serializer),
+                  'token': self.token}
         params.update(kwargs)
         return self._get('query', params=params)
 
@@ -409,11 +434,11 @@ class TodoistAPI(object):
         """
         Updates the user's notification settings.
         """
-        return self._get('update_notification_setting',
-                         params={'token': self.token,
-                                 'notification_type': notification_type,
-                                 'service': service,
-                                 'dont_notify': dont_notify})
+        return self._post('update_notification_setting',
+                          data={'token': self.token,
+                                'notification_type': notification_type,
+                                'service': service,
+                                'dont_notify': dont_notify})
 
     def get_all_completed_items(self, **kwargs):
         """
@@ -423,6 +448,36 @@ class TodoistAPI(object):
         params.update(kwargs)
         return self._get('get_all_completed_items', params=params)
 
+    def get_completed_items(self, project_id, **kwargs):
+        """
+        Returns a project's completed items.
+        """
+        params = {'token': self.token,
+                  'project_id': project_id}
+        params.update(kwargs)
+        return self._get('get_completed_items', params=params)
+
+    def get_uploads(self, **kwargs):
+        """
+        Returns all user's uploads.
+
+        kwargs:
+            limit: (int, optional) number of results (1-50)
+            last_id: (int, optional) return results with id<last_id
+        """
+        params = {'token': self.token}
+        params.update(kwargs)
+        return self._get('uploads/get', params=params)
+
+    def delete_upload(self, file_url):
+        """
+        Delete upload.
+
+        param file_url: (str) uploaded file URL
+        """
+        params = {'token': self.token, 'file_url': file_url}
+        return self._get('uploads/delete', params=params)
+
     def add_item(self, content, **kwargs):
         """
         Adds a new task.
@@ -437,7 +492,7 @@ class TodoistAPI(object):
         """
         Appends a request to the queue, to share a project with a user.
         """
-        item = {
+        cmd = {
             'type': 'share_project',
             'temp_id': self.generate_uuid(),
             'uuid': self.generate_uuid(),
@@ -446,15 +501,15 @@ class TodoistAPI(object):
                 'email': email,
             },
         }
-        item['args'].update(kwargs)
-        self.queue.append(item)
+        cmd['args'].update(kwargs)
+        self.queue.append(cmd)
 
     def delete_collaborator(self, project_id, email):
         """
         Appends a request to the queue, to delete a collaborator from a shared
         project.
         """
-        item = {
+        cmd = {
             'type': 'delete_collaborator',
             'uuid': self.generate_uuid(),
             'args': {
@@ -462,20 +517,20 @@ class TodoistAPI(object):
                 'email': email,
             },
         }
-        self.queue.append(item)
+        self.queue.append(cmd)
 
     def take_ownership(self, project_id):
         """
         Appends a request to the queue, take ownership of a shared project.
         """
-        item = {
+        cmd = {
             'type': 'take_ownership',
             'uuid': self.generate_uuid(),
             'args': {
                 'project_id': project_id,
             },
         }
-        self.queue.append(item)
+        self.queue.append(cmd)
 
     # Auxiliary
     def get_project(self, project_id):
@@ -563,8 +618,6 @@ class TodoistAPI(object):
     def __repr__(self):
         name = self.__class__.__name__
         unsaved = '*' if len(self.queue) > 0 else ''
-        if self.seq_no == 0:
-            email = '<not synchronized>'
-        else:
-            email = repr(self.state['User']['email'])
-        return '%s%s(%s)' % (name, unsaved, email)
+        email = self.user.get('email')
+        email_repr = repr(email) if email else '<not synchronized>'
+        return '%s%s(%s)' % (name, unsaved, email_repr)
