@@ -22,12 +22,11 @@ class LiveNotificationsManager(Manager, AllMixin, SyncMixin):
         Sets in the local state the last notification read, and appends the
         equivalent request to the queue.
         """
-        self.state[self.state_name] = seq_no
-        item = {
+        cmd = {
             'type': 'live_notifications_mark_as_read',
             'uuid': self.api.generate_uuid(),
             'args': {
                 'seq_no': seq_no,
             },
         }
-        self.queue.append(item)
+        self.queue.append(cmd)
